SPELLWATCH_TYPE_SPELL = 1;
SPELLWATCH_TYPE_SELFBUFF = 2;
SPELLWATCH_TYPE_SELFDEBUFF = 3;
SPELLWATCH_TYPE_TARGETBUFF = 4;
SPELLWATCH_TYPE_TARGETDEBUFF = 5;

local options = {
	type='group',
	args = {
		config = {
			name = "config",
			type = 'group',
			desc = "Configuration options",
			args = {
				death = {
					name = "death",
					type = 'toggle',
					desc = "Remove timers when target dies",
					get = function () return SpellWatch.db.profile.config.death; end,
					set = function () SpellWatch.db.profile.config.death = not SpellWatch.db.profile.config.death; end,
				},
				lost = {
					name = "lost",
					type = 'toggle',
					desc = "Remove timers when the buff/debuff is lost",
					get = function () return SpellWatch.db.profile.config.lost; end,
					set = function () SpellWatch.db.profile.config.lost = not SpellWatch.db.profile.config.lost; end,
				},
				resist = {
					name = "resist",
					type = 'toggle',
					desc = "Remove timers when the spell/ability is resisted/dodged/parried/blocked/immune",
					get = function () return SpellWatch.db.profile.config.resist; end,
					set = function () SpellWatch.db.profile.config.resist = not SpellWatch.db.profile.config.resist; end,
				},
				fade = {
					name = "fade",
					type = 'range',
					desc = "Seconds timers take to fade out",
					min = 0,
					max = 5,
					step = 0.1,
					get = function () return SpellWatch.db.profile.config.fade; end,
					set = function (value) SpellWatch.db.profile.config.fade = value; end,
				},
				colorfade = {
					name = "colorfade",
					type = 'toggle',
					desc = "Fade to red as a timer nears completion",
					get = function () return SpellWatch.db.profile.config.colorfade; end,
					set = function () SpellWatch.db.profile.config.colorfade = not SpellWatch.db.profile.config.colorfade; end,
				},
				combatclear = {
					name = "combatclear",
					type = 'toggle',
					desc = "Clear all timers when exiting combat, excluding self buffs/debuffs and timers on non-NPCs.",
					get = function () return SpellWatch.db.profile.config.combatclear; end,
					set = function () SpellWatch.db.profile.config.combatclear = not SpellWatch.db.profile.config.combatclear; end,
				},
				texture = {
					name = "texture",
					type = 'text',
					desc = "Timer bar texture",
					get = function () return SpellWatch.db.profile.config.texture; end,
					set = function (text) SpellWatch.db.profile.config.texture = text; end,
					validate = {"banto", "cilo", "glaze", "smooth", "solid"},
				},
			},
		},
		menu = {
			name = "menu",
			type = 'execute',
			desc = "Show the timer configuration window",
			func = function() SpellWatchMenuFrame:Show(); end
		},
	},
}

for i=1,3 do
    options.args["anchor"..i] = {
			name = "anchor"..i,
			type = 'group',
			desc = "Anchor "..i.." Settings",
			args = {
				show = {
					name = "show",
					type = 'toggle',
					desc = "Show/hide the anchor",
					get  = function () return getglobal("SpellWatchAnchor"..i.."DragBar"):IsVisible(); end,
					set  = function ()
							if getglobal("SpellWatchAnchor"..i.."DragBar"):IsVisible() then
								getglobal("SpellWatchAnchor"..i.."DragBar"):Hide();
							else
								getglobal("SpellWatchAnchor"..i.."DragBar"):Show();
							end
						end
				},
				scale = {
					name = "scale",
					type = 'range',
					desc = "Set the scale for the timers",
					min = 0.5,
					max = 2.0,
					step = 0.05,
					isPercent = true,
					get = function () return SpellWatch.db.profile["anchor"..i].scale; end,
					set = function (value) SpellWatch.db.profile["anchor"..i].scale = value; end,
				},
				width = {
					name = "width",
					type = 'range',
					desc = "Timer bar width",
					min = 50, 
					max = 400,
					step = 1,
					get = function () return SpellWatch.db.profile["anchor"..i].width; end,
					set = function (text) SpellWatch.db.profile["anchor"..i].width = text; end,
				},
				height = {
					name = "height",
					type = 'range',
					desc = "Timer bar height",
					min = 8, 
					max = 30,
					step = 1,
					get = function () return SpellWatch.db.profile["anchor"..i].height; end,
					set = function (text) SpellWatch.db.profile["anchor"..i].height = text; end,
				},
				font = {
					name = "font",
					type = 'range',
					desc = "Set the font size for the timers",
					min = 10,
					max = 20,
					step = 1,
					get = function () return SpellWatch.db.profile["anchor"..i].font; end,
					set = function (value) SpellWatch.db.profile["anchor"..i].font = value; end,
				},
				growup = {
					name = "growup",
					type = 'toggle',
					desc = "Grow timers up instead of down from the anchor",
					get = function () return SpellWatch.db.profile["anchor"..i].growup; end,
					set = function () SpellWatch.db.profile["anchor"..i].growup = not SpellWatch.db.profile["anchor"..i].growup;
							SpellWatch:SetCandyBarGroupGrowth("SpellWatchTimers"..i, SpellWatch.db.profile["anchor"..i].growup or false);
						  end,
				},
				vertspacing = {
					name = "vertspacing",
					type = 'range',
					min = 0,
					max = 30,
					step = 1,
					desc = "The vertical spacing between the bars of the group",
					get = function () return SpellWatch.db.profile["anchor"..i].vertspacing; end,
					set = function (value) SpellWatch.db.profile["anchor"..i].vertspacing = value; 
					        SpellWatch:SetCandyBarGroupVerticalSpacing("SpellWatchTimers"..i, SpellWatch.db.profile["anchor"..i].vertspacing or 0);      
					      end,
				},
			},
		};
end;
		
SpellWatch = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0", "CandyBar-2.0");
SpellWatch:RegisterChatCommand({"/spellwatch", "/sw"}, options);
SpellWatch:RegisterDB("SpellWatchDB");
SpellWatch:RegisterDefaults("profile", {
		anchor1 = { scale = 1.0, font = 12, height = nil, width = nil },
		anchor2 = { scale = 1.0, font = 12, height = nil, width = nil },
		anchor3 = { scale = 1.0, font = 12, height = nil, width = nil },
		config  = {	texture = "smooth", fade = 0.5, resist = true, colorfade = false, combatclear = false },
		timers = { }
	});

local Colors = AceLibrary("PaintChips-2.0");

-- Convert a hex value into three floats
SpellWatch.h2f = function (hex)
	hex = hex or "";
	local r, g, b = string.sub(hex,1,2), string.sub(hex,3,4), string.sub(hex,5,6);
	return (tonumber(r,16) or 0)/255, (tonumber(g,16) or 0)/255, (tonumber(b,16) or 0)/255;
end

-- Convert three floats into a hex value
SpellWatch.f2h = function (r, g, b)
	return string.format("%02X%02X%02X", floor(r*255), floor(g*255), floor(b*255));
end

function SpellWatch:OnInitialize()

	-- Set up variables
	self.timesinceupdate = 0;
	self.updatefreq = 0.1;
	self.cast = { };
	self.active = { };

	SpellWatchMenuFrameDragBarTitle:SetText("SpellWatch "..self.version);

	-- ADD TEXTURES HERE
	self.texture = { };
	self.texture["banto" ] = "Interface\\AddOns\\SpellWatch\\Textures\\banto.tga";
	self.texture["cilo"  ] = "Interface\\AddOns\\SpellWatch\\Textures\\cilo.tga";
	self.texture["glaze" ] = "Interface\\AddOns\\SpellWatch\\Textures\\glaze.tga";
	self.texture["smooth"] = "Interface\\AddOns\\SpellWatch\\Textures\\smooth.tga";
	self.texture["solid" ] = "Interface\\AddOns\\SpellWatch\\Textures\\solid.tga";

	-- Set up CandyBar groups
	self:RegisterCandyBarGroup("SpellWatchTimers1");
	self:SetCandyBarGroupPoint("SpellWatchTimers1", "CENTER", "SpellWatchAnchor1", "CENTER", 0, 0);
	self:SetCandyBarGroupGrowth("SpellWatchTimers1", self.db.profile.anchor1.growup or false);
    self:SetCandyBarGroupVerticalSpacing("SpellWatchTimers1", self.db.profile.anchor1.vertspacing or 0);
    
	self:RegisterCandyBarGroup("SpellWatchTimers2");
	self:SetCandyBarGroupPoint("SpellWatchTimers2", "CENTER", "SpellWatchAnchor2", "CENTER", 0, 0);
	self:SetCandyBarGroupGrowth("SpellWatchTimers2", self.db.profile.anchor2.growup or false);
    self:SetCandyBarGroupVerticalSpacing("SpellWatchTimers2", self.db.profile.anchor2.vertspacing or 0);
    
	self:RegisterCandyBarGroup("SpellWatchTimers3");
	self:SetCandyBarGroupPoint("SpellWatchTimers3", "CENTER", "SpellWatchAnchor3", "CENTER", 0, 0);
	self:SetCandyBarGroupGrowth("SpellWatchTimers3", self.db.profile.anchor3.growup or false);
	self:SetCandyBarGroupVerticalSpacing("SpellWatchTimers3", self.db.profile.anchor3.vertspacing or 0);

end

function SpellWatch:OnEnable()
	-- trigger events
	self:RegisterEvent("UNIT_SPELLCAST_SENT",							"EventSpellCast");
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED",						"EventSpellSucceeded");

	self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS",			"EventSelfBuff");
	self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_SELF_DAMAGE",			"EventSelfDebuff");

	self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_CREATURE_BUFFS",		"EventTargetBuff");
	self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_BUFFS",	"EventTargetBuff");
	self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_BUFFS", 	"EventTargetBuff");
	self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_PARTY_BUFFS",			"EventTargetBuff");

	self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_CREATURE_DAMAGE",		"EventTargetDebuff");
	self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_DAMAGE",	"EventTargetDebuff");
	self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_DAMAGE",	"EventTargetDebuff");
	self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_PARTY_DAMAGE",			"EventTargetDebuff");
	
	self:RegisterEvent("CHAT_MSG_COMBAT_SELF_HITS",						"EventMelee");

	-- cancel events
	self:RegisterEvent("PLAYER_REGEN_ENABLED",							"EventRegenEnabled");
	self:RegisterEvent("CHAT_MSG_COMBAT_FRIENDLY_DEATH",				"EventDeath");
	self:RegisterEvent("CHAT_MSG_COMBAT_HOSTILE_DEATH",					"EventDeath");
	self:RegisterEvent("CHAT_MSG_SPELL_AURA_GONE_SELF",					"EventAuraGoneSelf");
	self:RegisterEvent("CHAT_MSG_SPELL_AURA_GONE_PARTY",				"EventAuraGoneOther");
	self:RegisterEvent("CHAT_MSG_SPELL_AURA_GONE_OTHER",				"EventAuraGoneOther");
	self:RegisterEvent("CHAT_MSG_SPELL_BREAK_AURA",						"EventBreakAura");
	self:RegisterEvent("CHAT_MSG_SPELL_SELF_DAMAGE",					"EventResist");

end

function SpellWatch:OnDisable()
    -- Called when the addon is disabled
end

function SpellWatch:OnUpdate(elapsed)

	self.timesinceupdate = self.timesinceupdate + elapsed;
	
	if self.timesinceupdate > self.updatefreq then
		self.timesinceupdate = 0;
		
		self:UpdateCandyBarGroup("SpellWatchTimers1");
		self:UpdateCandyBarGroup("SpellWatchTimers2");
		self:UpdateCandyBarGroup("SpellWatchTimers3");
	end

end

------------------------------
--      TRIGGER EVENTS      --
------------------------------

function SpellWatch:EventSpellCast(player, spell, rank, target)

	--self:Print("UNIT_SPELLCAST_SENT : "..player..", "..spell..", "..rank..", "..target);

	self.cast.name = spell;
	self.cast.rank = tonumber(rank:match("Rank (%d+)"));
	self.cast.target = (target ~= "") and target or nil;
	self.cast.cp = (GetComboPoints() > 0) and GetComboPoints() or nil;
	
	-- check whether you're targeting target, focus, or mouseover
	if UnitName("target") == self.cast.target then
		self.cast.targetsex = UnitSex("target");
		self.cast.targetlevel = UnitLevel("target");
		self.cast.targeticon = GetRaidTargetIndex("target");
		self.cast.targetpc = UnitIsPlayer("target");
	elseif UnitName("focus") == self.cast.target then
		self.cast.targetsex = UnitSex("focus");
		self.cast.targetlevel = UnitLevel("focus");
		self.cast.targeticon = GetRaidTargetIndex("focus");
		self.cast.targetpc = UnitIsPlayer("focus");
	elseif UnitName("mouseover") == self.cast.target then
		self.cast.targetsex = UnitSex("mouseover");
		self.cast.targetlevel = UnitLevel("mouseover");
		self.cast.targeticon = GetRaidTargetIndex("mouseover");
		self.cast.targetpc = UnitIsPlayer("mouseover");
	end

end

function SpellWatch:EventSpellSucceeded(unit, spell, rank)

	--self:Print("UNIT_SPELLCAST_SUCCEEDED : "..unit..", "..spell..", "..rank);

	-- check args
	if unit ~= "player" then return; end
	if spell ~= self.cast.name then return; end

	for name, entry in pairs(self.db.profile.timers) do
		if ( entry.type == SPELLWATCH_TYPE_SPELL ) and
		(( entry.spell == self.cast.name ) and (( entry.rank == nil ) or ( entry.rank == self.cast.rank ))) then
			self:TriggerSpellCast(name, self.cast.rank, self.cast.target, self.cast.targetpc, self.cast.targetsex, self.cast.targetlevel, self.cast.targeticon, self.cast.cp);
		end
	end

	-- clear out cast data
	self.cast.name = nil;
	self.cast.rank = nil;
	self.cast.target = nil;
	self.cast.targetsex = nil;
	self.cast.targetlevel = nil;
	self.cast.targeticon = nil;
	self.cast.targetpc = nil;
	self.cast.cp = nil;

end

function SpellWatch:EventSelfBuff(message)

	if not message then return; end

	local buff = string.match(message, "You gain (.+)%.");

	if not buff then return; end

	for name, entry in pairs(self.db.profile.timers) do
		if ( entry.type == SPELLWATCH_TYPE_SELFBUFF ) and ( entry.spell == buff ) then
			self:TriggerBuff(name, "You");
		end
	end

end

function SpellWatch:EventSelfDebuff(message)

	if not message then return; end
	
	local debuff = string.match(message, "You are afflicted by (.+)%.");

	if not debuff then return; end

	for name, entry in pairs(self.db.profile.timers) do
		if ( entry.type == SPELLWATCH_TYPE_SELFDEBUFF ) and ( entry.spell == debuff ) then
			self:TriggerBuff(name, "You");
		end
	end

end

function SpellWatch:EventTargetBuff(message)

	if not message then return; end

	local target, buff = string.match(message, "(.+) gains (.+)%.");

	if not target or not buff then return; end
	if not UnitExists("target") or ( UnitName("target") ~= target ) then return; end

	for name, entry in pairs(self.db.profile.timers) do
		if ( entry.type == SPELLWATCH_TYPE_TARGETBUFF ) and ( entry.spell == buff ) then
			self:TriggerBuff(name, target);
		end
	end

end

function SpellWatch:EventTargetDebuff(message)

	if not message then return; end
	
	local target, debuff = string.match(message, "(.+) is afflicted by (.+)%.");

	if not target or not debuff then return; end

	if not UnitExists("target") or ( UnitName("target") ~= target ) then return; end

	for name, entry in pairs(self.db.profile.timers) do
		if ( entry.type == SPELLWATCH_TYPE_TARGETDEBUFF ) and ( entry.spell == debuff ) then
			self:TriggerBuff(name, target);
		end
	end

end

function SpellWatch:EventMelee(message)

	if not message then return; end

	local target = string.match(message, "You hit (.+) for %d+%.");
	target = target or string.match(message, "You crit (.+) for %d+%.");

	if not target then return; end

	local targetsex, targetlevel, targeticon;

	if UnitName("target") ~= target then
		self:Print("Warning! Target name differs from melee strike in log!");
	end

	targetsex = UnitSex("target");
	targetlevel = UnitLevel("target");
	targeticon = GetRaidTargetIndex("target");
	
	-- restart any active refresh-on-melee timers with the same target
	for name, active in pairs(self.active) do
		if (( active.target == target ) and ( active.targetsex == targetsex ) and
		( active.targetlevel == targetlevel ) and ( active.targeticon == targeticon )) and ( active.config.romelee ) then
			self:SetCandyBarTimeLeft(name, active.duration);
		end
	end

end

-----------------------------
--      CANCEL EVENTS      --
-----------------------------

function SpellWatch:EventRegenEnabled()
	
	if not self.db.profile.config.combatclear then return; end
	
	local active;
	for _, active in pairs(self.active) do
		if active.target and ( not active.targetpc ) and (( active.config.type ~= SPELLWATCH_TYPE_SELFBUFF ) and ( active.config.type ~= SPELLWATCH_TYPE_SELFDEBUFF)) then
			self:UnregisterTimer(active.bar);
		end
	end
	
end

function SpellWatch:EventDeath(message)

	if not self.db.profile.config.death then return; end

	local target = string.match(message, "(.+) dies");

	if not target then return; end

	-- cancel any active timers with matching target
	local active;
	for _, active in pairs(self.active) do
		if active.target and ( target == active.target ) and (( not active.config.nocancel ) or ( active.targetpc )) then
			self:UnregisterTimer(active.bar);
		end
	end

end

function SpellWatch:EventAuraGoneSelf(message)

	if not self.db.profile.config.lost then return; end

	local buff = string.match(message, "(.+) fades from you%.");

	if not buff then return; end

	-- cancel any active self buff/debuff timers with matching name
	local active;
	for _, active in pairs(self.active) do
		if ( buff == active.config.spell ) and ( not active.config.nocancel ) then
			self:UnregisterTimer(active.bar);
		end
	end

end

function SpellWatch:EventAuraGoneOther(message)

	if not self.db.profile.config.lost then return; end

	local buff, target = string.match(message, "(.+) fades from (.+)%.");

	if not buff or not target then return; end

	-- cancel any active target buff/debuff timers with matching name & target
	local active;
	for _, active in pairs(self.active) do
		if active.target and ( target == active.target ) and ( buff == active.config.spell ) and ( not active.config.nocancel ) then
			self:UnregisterTimer(active.bar);
		end
	end

end

function SpellWatch:EventBreakAura(message)

	if not self.db.profile.config.lost then return; end

	local target, buff;
	
	target, buff = string.match(message, "(.+)'s (.+) is removed%.");

	if target and buff then
		-- cancel any active targeted timers with matching name & target
		local active;
		for _, active in pairs(self.active) do
			if active.target and ( target == active.target ) and ( buff == active.config.spell ) then
				self:UnregisterTimer(active.bar);
			end
		end
	else
		buff = string.match(message, "Your (.+) is removed%.");
		if not buff then return; end

		-- cancel any active self buff/debuff timers with matching name
		local active;
		for _, active in pairs(self.active) do
			if ((active.config.type == SPELLWATCH_TYPE_SELFBUFF) or (active.config.type == SPELLWATCH_TYPE_SELFDEBUFF))
			and ( buff == active.config.spell ) and ( not active.config.nocancel ) then
				self:UnregisterTimer(active.bar);
			end
		end
	end

end

function SpellWatch:EventResist(message)
	
	if not self.db.profile.config.resist then return; end
	
	local spell, target;

	spell, target = string.match(message, "Your (.+) was resisted by (.+)%.");
	if not spell or not target then spell, target = string.match(message, "Your (.+) misse[sd] (.+)%."); end
	if not spell or not target then spell, target = string.match(message, "Your (.+) [wi]a?s dodged by (.+)%."); end
	if not spell or not target then spell, target = string.match(message, "Your (.+) [wi]a?s parried by (.+)%."); end
	if not spell or not target then spell, target = string.match(message, "Your (.+) was blocked by (.+)%."); end
	if not spell or not target then spell, target = string.match(message, "Your (.+) failed%. (.+) [wi]a?s immune%."); end
	if not spell or not target then target, spell = string.match(message, "(.+) is immune to your (.+)%."); end
	if not spell or not target then spell, target = string.match(message, "Your (.+) [wi]a?s evaded by (.+)%."); end
	if not spell or not target then spell, target = string.match(message, "Your (.+) [wi]a?s reflected back by (.+)%."); end
	if not spell or not target then return; end
	
	-- cancel any active targeted timers with matching name & target
	local active;
	for _, active in pairs(self.active) do
		if active.target and ( target == active.target ) and ( spell == active.config.spell ) then
			self:UnregisterTimer(active.bar);
		end
	end

end

-------------------------
--     Timer Setup     --
-------------------------

function SpellWatch:TriggerSpellCast(name, rank, target, targetpc, targetsex, targetlevel, targeticon, cp)

	local config = self.db.profile.timers[name];

	local timer = { };
	timer.name = name;
	timer.config = config;
	timer.bar = name..math.random(0, 7625597);
	timer.target = config.targeted and target;
	timer.targetsex = config.targeted and targetsex;
	timer.targetlevel = config.targeted and targetlevel;
	timer.targeticon = config.targeted and targeticon;
	timer.targetpc = targetpc;
	
	local duration 
	if ( cp and cp > 0 ) then
		-- Combo point timer modifications.
		duration = config.duration + (config.percp or 0)*(cp and (cp > 2) and (cp - 1) or 0);
	else
		-- Rank modifications. Can only be used for spells which always gain duration each rank, ie: Banish, Fear, Howl of Terror
		duration = config.duration + (config.percp or 0)*(rank and (rank - 1) or 0)
	end
		
	local label = string.gsub(config.label, "$n", config.spell);
	label = string.gsub(label, "$t", timer.target or "<no target>");
	
	timer.duration = duration;

	-- Iterate through active timers
	local active;
	for _, active in pairs(self.active) do

		-- if group matches
		if config.group and config.group ~= "" and ( config.group == active.config.group ) then
			-- if targeted, cancel other timers of same group and target
			-- if not targeted, cancel other timers regardless of target
			if not config.targeted or (( target == active.target ) and ( targetsex == active.targetsex )
				and ( targetlevel == active.targetlevel ) and ( targeticon == active.targeticon )) then
				self:UnregisterTimer(active.bar);
			end

		-- if same spell
		elseif ( name == active.name ) then
			-- if unique, cancel other instances of this timer
			-- if targeted on same target, cancel
			if config.unique or (config.targeted and (( target == active.target ) and ( targetsex == active.targetsex )
				and ( targetlevel == active.targetlevel ) and ( targeticon == active.targeticon ))) then
				self:UnregisterTimer(active.bar);
			end

		end

	end

	-- register new timer
	self:RegisterTimer(timer, duration, label, config.icon, config.color);

end

function SpellWatch:TriggerBuff(name, target)

	local config = self.db.profile.timers[name];

	local timer = { };
	timer.name = name;
	timer.config = config;
	timer.bar = name..math.random(0, 7625597);
	timer.target = target;

	local label = string.gsub(config.label, "$n", config.spell);
	label = string.gsub(label, "$t", timer.target or "<no target>");

	-- Iterate through active timers
	local active;
	for _, active in pairs(self.active) do

		-- cancel other timers of same group and target
		if config.group and config.group ~= "" and ( config.group == active.config.group ) and ( target == active.target ) then
				self:UnregisterTimer(active.bar);

		-- if same spell
		elseif ( name == active.name ) then
			-- if unique, cancel other instances of this timer
			-- if on same target, cancel
			if config.unique or ( target == active.target ) then
				self:UnregisterTimer(active.bar);
			end
		end

	end

	-- register new timer
	self:RegisterTimer(timer, config.duration, label, config.icon, config.color);

end




-- Registers a new timer
function SpellWatch:RegisterTimer(timer, duration, label, icon, color)

	if duration <= 0 then return; end

	self.active[timer.bar] = timer;
	Colors:RegisterHex(color);
	self:RegisterCandyBar(timer.bar, duration, label, icon, color, color, color, SpellWatch.db.profile.config.colorfade and "red" or color);
	self:RegisterCandyBarWithGroup(timer.bar, "SpellWatchTimers"..(timer.config.anchor or 1));
	self:SetCandyBarScale(timer.bar, self.db.profile["anchor"..(timer.config.anchor or 1)].scale);
	if self.db.profile["anchor"..timer.config.anchor].width then self:SetCandyBarWidth(timer.bar, self.db.profile["anchor"..timer.config.anchor].width); end
	if self.db.profile["anchor"..timer.config.anchor].height then self:SetCandyBarHeight(timer.bar, self.db.profile["anchor"..timer.config.anchor].height); end
	self:SetCandyBarFontSize(timer.bar, self.db.profile["anchor"..(timer.config.anchor or 1)].font);
	self:SetCandyBarTexture(timer.bar, self.texture[self.db.profile.config.texture]);
	self:SetCandyBarFade(timer.bar, self.db.profile.config.fade);
	self:SetCandyBarCompletion(timer.bar, self.UnregisterTimer, self, timer.bar);
	self:SetCandyBarOnClick(timer.bar, self.TimerOnClick);
	self:StartCandyBar(timer.bar, true);

	-- announce
	if timer.config.announce then
		UIErrorsFrame:AddMessage(label);
	end

end

-- Unregisters an existing timer
function SpellWatch:UnregisterTimer(bar)

	self:StopCandyBar(bar);
	self.active[bar] = nil;

end

-- Timer on click function
function SpellWatch.TimerOnClick(bar, button)

	if ( button == "RightButton" ) then
		SpellWatch:StopCandyBar(bar);
		SpellWatch.active[bar] = nil;
	end

end
